# -*- coding: UTF-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

"""
GUI test tool and automation framework that uses Accessibility (a11y) technologies to communicate
with desktop applications.
"""

__author__ = """Zack Cerza <zcerza@redhat.com>,
Ed Rousseau <rousseau@redhat.com>,
David Malcolm <dmalcolm@redhat.com>,
Vita Humpa <vhumpa@redhat.com>"""
__version__ = "0.9.11"
__copyright__ = "Copyright © 2005-2017 Red Hat, Inc."
__license__ = "GPL"

__all__ = ("config", "distro", "dump", "errors", "i18n", "logging", "path", "predicate",
           "procedural", "rawinput", "sessions", "tc", "tree", "utils", "version", "wrapped")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
import unittest
import dogtail.config

"""
Unit tests for the dogtail.config class
"""


class TestConfiguration(unittest.TestCase):

    def test_get_set_all_properties(self):
        for option in list(dogtail.config.config.defaults.keys()):
            print("Setting config.%s property" % str(option))
            value = ''
            if 'Dir' in option:
                value = '/tmp/dogtail/'  # Special value for dir-related properties
            dogtail.config.config.__setattr__(option, value)
            self.assertEqual(dogtail.config.config.__getattr__(option), value)

    def test_default_directories_created(self):
        import os.path
        self.assertEqual(
            os.path.isdir(dogtail.config.config.scratchDir), True)
        self.assertEqual(os.path.isdir(dogtail.config.config.logDir), True)
        self.assertEqual(os.path.isdir(dogtail.config.config.dataDir), True)

    def test_set(self):
        self.assertRaises(AttributeError, setattr, dogtail.config.config, 'nosuchoption', 42)

    def test_get(self):
        self.assertRaises(AttributeError, getattr, dogtail.config.config, 'nosuchoption')

    def helper_create_directory_and_set_option(self, path, property_name):
        import os.path
        if os.path.isdir(path):
            import shutil
            shutil.rmtree(path)
        dogtail.config.config.__setattr__(property_name, path)
        self.assertEqual(os.path.isdir(path), True)

    def test_create_scratch_directory(self):
        new_folder = "/tmp/dt"
        self.helper_create_directory_and_set_option(new_folder, 'scratchDir')

    def test_create_data_directory(self):
        new_folder = "/tmp/dt_data"
        self.helper_create_directory_and_set_option(new_folder, 'dataDir')

    def test_create_log_directory(self):
        new_folder = "/tmp/dt_log"
        self.helper_create_directory_and_set_option(new_folder, 'logDir')

    def test_load(self):
        dogtail.config.config.load({'actionDelay': 2.0})
        self.assertEqual(dogtail.config.config.actionDelay, 2.0)

    def test_reset(self):
        default_actionDelay = dogtail.config.config.defaults['actionDelay']
        dogtail.config.config.actionDelay = 2.0
        dogtail.config.config.reset()
        self.assertEqual(dogtail.config.config.actionDelay, default_actionDelay)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
import unittest


class GtkDemoTest(unittest.TestCase):
    """
    TestCase subclass which handles bringing up and shutting down gtk3-demo as a fixture.  Used for writing other test
    cases.
    """

    def setUp(self):
        import dogtail.config
        dogtail.config.config.logDebugToStdOut = True
        dogtail.config.config.logDebugToFile = False
        import dogtail.utils
        self.pid = dogtail.utils.run('gtk3-demo')
        self.app = dogtail.tree.root.application('gtk3-demo')

    def tearDown(self):
        import os
        import signal
        import time
        os.kill(self.pid, signal.SIGKILL)
        os.system('killall gtk3-demo-application > /dev/null 2>&1')
        # Sleep just enough to let the app actually die.
        # AT-SPI doesn't like being hammered too fast.
        time.sleep(0.5)

    def runDemo(self, demoName, retry=True):
        """
        Click on the named demo within the gtk3-demo app.
        """
        tree = self.app.child(roleName="tree table")
        tree.child(demoName, retry=retry).doActionNamed('activate')


def trap_stdout(function, args=None):
    """
    Grab stdout output during function execution
    """

    import sys
    from io import StringIO

    saved_stdout = sys.stdout
    try:
        out = StringIO()
        sys.stdout = out
        if type(args) is dict:
            function(**args)
        elif args:
            function(args)
        else:
            function()
        output = out.getvalue().strip()
    finally:
        sys.stdout = saved_stdout
    return output



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
import dogtail.predicate
import dogtail.tree
import dogtail.i18n
import os.path
import dogtail.config
import dogtail.version
import dogtail.utils
from gtkdemotest import GtkDemoTest, trap_stdout
import unittest
import time

"""
Unit tests for the dogtail.procedural API
"""
__author__ = "Zack Cerza <zcerza@redhat.com>"

dogtail.config.config.logDebugToFile = False
dogtail.config.config.logDebugToStdOut = True


class TestScreenshot(GtkDemoTest):

    def make_expected_and_compare(self, actual_path, jpg_tolerance=None):
        extension = actual_path.split('.')[-1]
        expected_path = actual_path.replace(extension, "expected." + extension)

        import os
        os.system("gnome-screenshot -f %s" % expected_path)

        command = ["compare", "-metric", "MAE",
                   actual_path, expected_path, "output"]
        import subprocess
        p = subprocess.Popen(command, stderr=subprocess.PIPE)
        output, error = p.communicate()

        import re
        m = re.search(r"\((.*)\)", str(error))
        self.assertTrue(0.1 >= float(m.group(1)))

    def test_screenshot_incorrect_timestamp(self):
        self.assertRaises(TypeError, dogtail.utils.screenshot, "timeStamp", None)

    def test_screenshot_default(self):
        actual_path = dogtail.utils.screenshot()
        self.make_expected_and_compare(actual_path)

    def test_screenshot_basename(self):
        actual_path = dogtail.utils.screenshot("basename")
        self.make_expected_and_compare(actual_path)

    def test_screenshot_no_time_stamp(self):
        actual_path = dogtail.utils.screenshot(timeStamp=False)
        self.make_expected_and_compare(actual_path)

    def test_screenshot_jpeg(self):
        actual_path = dogtail.utils.screenshot("basename.jpg")
        self.make_expected_and_compare(actual_path, jpg_tolerance=True)

    def test_screenshot_unknown_format(self):
        self.assertRaises(ValueError, dogtail.utils.screenshot, "basename.dat")


class TestRun(unittest.TestCase):

    def setUp(self):
        self.pid = None

    def tearDown(self):
        import os
        import signal
        if self.pid:
            os.kill(self.pid, signal.SIGKILL)
        os.system('killall gtk3-demo-application > /dev/null 2>&1')
        # Sleep just enough to let the app actually die.
        # AT-SPI doesn't like being hammered too fast.
        time.sleep(0.5)

    def test_run(self):
        self.pid = dogtail.utils.run('gtk3-demo')
        dogtail.tree.root.application('gtk3-demo')

    def test_run_wrong(self):
        self.pid = None
        with self.assertRaises(OSError):
            self.pid = dogtail.utils.run('gtk3-virtual-nonexisting-demo')

    def test_run_dumb(self):
        self.pid = dogtail.utils.run('gtk3-demo', dumb=True)
        dogtail.tree.root.application('gtk3-demo')


class TestDelay(unittest.TestCase):

    def test_doDelay_implicit(self):
        dogtail.utils.config.defaultDelay = 2.0
        start = time.time()
        dogtail.utils.doDelay()
        self.assertTrue(time.time() - start >= 2.0)

    def test_doDelay_explicit(self):
        dogtail.utils.config.defaultDelay = 1.0
        start = time.time()
        dogtail.utils.doDelay(2.0)
        self.assertTrue(time.time() - start >= 2.0)

    def test_doDelay_logger(self):
        dogtail.utils.config.defaultDelay = 2.0
        dogtail.utils.config.debugSleep = True
        output = trap_stdout(dogtail.utils.doDelay).split()
        self.assertEqual(len(output), 3)
        self.assertEqual(float(output[2]), 2.0)


class TestA11Y(unittest.TestCase):

    def test_bail_when_a11y_disabled(self):
        self.assertRaises(SystemExit, dogtail.utils.bailBecauseA11yIsDisabled)

    def test_enable_a11y(self):
        dogtail.utils.enableA11y()


class TestLock(unittest.TestCase):

    def tearDown(self):
        os.system("rm -rf /tmp/dogtail-test.lock*")

    def test_set_unrandomized_lock(self):
        test_lock = dogtail.utils.Lock(lockname='dogtail-test.lock', randomize=False)
        self.assertEqual(test_lock.lockdir, "/tmp/dogtail-test.lock")
        self.assertFalse(os.path.isdir(test_lock.lockdir))
        test_lock.lock()
        self.assertTrue(os.path.isdir(test_lock.lockdir))
        test_lock.unlock()
        self.assertFalse(os.path.isdir(test_lock.lockdir))

    def test_double_lock(self):
        test_lock = dogtail.utils.Lock(lockname='dogtail-test.lock', randomize=False, unlockOnExit=True)
        test_lock.lock()
        with self.assertRaises(OSError):
            test_lock.lock()

    def test_double_unlock(self):
        test_lock = dogtail.utils.Lock(lockname='dogtail-test.lock', randomize=False)
        test_lock.lock()
        test_lock.unlock()
        with self.assertRaises(OSError):
            test_lock.unlock()

    def test_randomize(self):
        test_lock = dogtail.utils.Lock(lockname='dogtail-test.lock', randomize=True)
        self.assertIn("/tmp/dogtail-test.lock", test_lock.lockdir)
        self.assertFalse(os.path.isdir(test_lock.lockdir))
        test_lock.lock()
        self.assertTrue(os.path.isdir(test_lock.lockdir))
        test_lock.unlock()
        self.assertFalse(os.path.isdir(test_lock.lockdir))


class TestI18N(unittest.TestCase):

    def test_load_all_translations_for_language(self):
        dogtail.i18n.loadAllTranslationsForLanguage('en_US')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
import dogtail.config
import dogtail.predicate
import dogtail.tree
import dogtail.rawinput
from gtkdemotest import GtkDemoTest, trap_stdout
import pyatspi
import time
import unittest
import os

"""
Unit tests for the dogtail.Node class

Notes on pyunit (the "unittest" module):

Test classes are written as subclass of unittest.TestCase.
A test is a method of such a class, beginning with the string "test"

unittest.main() will run all such methods.  Use "-v" to get feedback on which tests are being run.
Tests are run in alphabetical order; all failure reports are gathered at the end.

setUp and tearDown are "magic" methods, called before and after each such
test method is run.
"""
__author__ = "Dave Malcolm <dmalcolm@redhat.com>"

dogtail.config.config.logDebugToFile = False


class TestNode(GtkDemoTest):

    """
    Unit tests for the the various synthesized attributes of a Node
    """

    def test_get_bogus(self):
        """
        Getting a non-existant attribute should raise an attribute error
        """
        self.assertRaises(AttributeError, getattr, self.app, "thisIsNotAnAttribute")

    # FIXME: should setattr for a non-existant attr be allowed?

    # 'name' (read-only string):
    def test_get_name(self):
        """
        Node.name of the gtk-demo app should be "gtk-demo"
        """
        self.assertEqual(self.app.name, 'gtk3-demo')

        self.assertEqual(dogtail.tree.root.name, 'main')

    def test_set_name(self):
        """
        Node.name should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "name", "hello world")

    # 'debugName' (string)
    def test_get_debugName(self):
        self.assertEqual(self.app.debugName, '"gtk3-demo" application')
        self.assertEqual(dogtail.tree.root.debugName, 'root')

    def test_set_debugName(self):
        self.app.debugName = "my application"
        self.assertEqual(self.app.debugName, 'my application')
        dogtail.tree.root.debugName = "my root"
        self.assertEqual(dogtail.tree.root.debugName, 'my root')

    # 'roleName' (read-only string):
    def test_get_roleName(self):
        """
        roleName of the gtk3-demo app should be "application"
        """
        self.assertEqual(self.app.roleName, 'application')

    def test_set_roleName(self):
        """
        Node.roleName should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "roleName", "hello world")

    # 'role' (read-only atspi role enum):
    def test_get_role(self):
        """
        Node.role for a gtk-demo app should be SPI_ROLE_APPLICATION
        """
        self.assertEqual(self.app.role, dogtail.tree.pyatspi.ROLE_APPLICATION)

    def test_set_role(self):
        """
        Node.role should be read-only
        """
        # FIXME should be AttributeError?
        self.assertRaises(RuntimeError, self.app.__setattr__, "role", pyatspi.Atspi.Role(1))

    # 'description' (read-only string):
    def test_get_description(self):
        # FIXME: can we get a more interesting test case here?
        self.assertEqual(self.app.description, "")

    def test_set_description(self):
        """
        Node.description should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "description", "hello world")

    # 'parent' (read-only Node instance):
    def test_get_parent(self):
        # the app has a parent if gnome-shell is used, so parent.parent is a
        # safe choice
        if [x for x in self.app.applications() if x.name == 'gnome-shell']:
            self.assertEqual(self.app.parent.parent, None)
        self.assertEqual(self.app.children[0].parent, self.app)

    def test_set_parent(self):
        """
        Node.parent should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "parent", None)

    # 'children' (read-only list of Node instances):
    def test_get_children(self):
        """
        A fresh gtk-demo app should have a single child: the window.
        """
        kids = self.app.children
        self.assertEqual(len(kids), 1)
        self.assertEqual(kids[0].name, "Application Class")
        self.assertEqual(kids[0].roleName, "frame")

    def test_set_children(self):
        """
        Node.children should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "children", [])

    def test_get_children_with_limit(self):
        haveWarnedAboutChildrenLimit = False
        print(haveWarnedAboutChildrenLimit)  # make pyflakes happy about it
        dogtail.config.config.childrenLimit = 1
        widget = self.app.child(roleName='tree table')
        self.assertEqual(len(widget.children), 1)

    #  combovalue (string):
    def test_get_combo_value(self):
        self.runDemo('Combo Boxes')
        try:
            wnd = self.app.child('Combo boxes', roleName='frame', retry=False)
        except dogtail.tree.SearchError:
            wnd = self.app.child('Combo Boxes', roleName='frame', retry=False)
        combo = wnd.child(roleName='combo box')
        self.assertEqual(combo.combovalue, 'dialog-warning')

    def test_get_URI_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            self.app.URI

    # 'text' (string):
    def test_set_text(self):
        """
        Use gtk-demo's text entry example to check that reading and writing
        Node.text works as expected
        """
        try:
            self.runDemo('Dialog and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs', roleName='frame', retry=False)
        except dogtail.tree.SearchError:
            self.runDemo('Dialogs and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs and Message Boxes', roleName='frame', retry=False)
        wnd.button('Interactive Dialog').click()
        dlg = self.app.dialog('Interactive Dialog')
        entry1 = dlg.child(label='Entry 1')
        entry2 = dlg.child(roleName='text')

        # Try reading the entries:
        self.assertEqual(entry1.text, "")
        self.assertEqual(entry2.text, "")

        # Set them...
        entry1.text = "hello"
        entry2.text = "world"

        # Ensure that they got set:
        self.assertEqual(entry1.text, "hello")
        self.assertEqual(entry2.text, "world")

        # and try again, searching for them again, to ensure it actually
        # affected the UI:
        self.assertEqual(dlg.child(label='Entry 1').text, "hello")
        self.assertEqual(dlg.child(label='Entry 2').text, "world")

        # Ensure app.text is None
        self.assertEqual(self.app.text, None)

        # Ensure a label's text is read-only as expected:
        # FIXME: this doesn't work; the label has no 'text'; it has a name.  we wan't a readonly text entry
        # label = dlg.child('Entry 1')
        # self.assertRaises(dogtail.tree.ReadOnlyError, label.text.__setattr__, "text", "hello world")

        # FIXME: should we assert that things are logged and delays are added?
        # FIXME: should have a test case involving the complex GtkTextView
        # widget

    def test_text_set_error(self):
        with self.assertRaises(AttributeError):
            self.app.text = 'something'

    def test_caretOffset(self):
        """
        Make sure the caret offset works as expected
        """
        try:
            self.runDemo('Dialog and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs', roleName='frame', retry=False)
        except dogtail.tree.SearchError:
            self.runDemo('Dialogs and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs and Message Boxes', roleName='frame', retry=False)
        entry1 = wnd.child(label='Entry 1')
        entry2 = wnd.child(roleName='text')

        # Try reading the entries:
        self.assertEqual(entry1.text, '')
        self.assertEqual(entry2.text, '')

        # Set them...
        s1 = "I just need a sentence"
        s2 = "And maybe a second one to be sure"
        entry1.text = s1
        entry2.text = s2

        # Make sure the caret offset is zero
        self.assertEqual(entry1.caretOffset, 0)
        self.assertEqual(entry2.caretOffset, 0)

        # Set the caret offset to something ridiculous
        entry1.caretOffset = len(s1 * 3)
        entry2.caretOffset = len(s2 * 3)

        # Make sure the caret offset only goes as far as the end of the string
        self.assertEqual(entry1.caretOffset, len(s1))
        self.assertEqual(entry2.caretOffset, len(s2))

        def splitByOffsets(node, string):
            # Verify the equality of node.text and string, word by word.
            # I realize this doesn't really test dogtail itself, but that could
            #   change in the future and I don't want to throw the code away.
            textIface = node.queryText()
            endOffset = -1  # We only set this now so the loop looks nicer
            startOffset = 0
            while startOffset != len(string):
                (text, startOffset, endOffset) = textIface.getTextAtOffset(
                    startOffset, pyatspi.TEXT_BOUNDARY_WORD_START)
                self.assertEqual(startOffset, string.find(text, startOffset, endOffset))
                startOffset = endOffset

        splitByOffsets(entry1, s1)
        splitByOffsets(entry2, s2)

    # 'combovalue' (read/write string):
    def test_comboValue(self):
        self.runDemo('Combo Boxes')
        try:
            wnd = self.app.child('Combo boxes', roleName='frame', retry=False)
        except dogtail.tree.SearchError:
            wnd = self.app.child('Combo Boxes', roleName='frame', retry=False)
        combo1 = wnd.child('Items with icons').child(roleName='combo box')
        combo1.combovalue = 'Clear'
        self.assertEqual(combo1.combovalue, 'edit-clear')

    # 'stateSet' (read-only StateSet instance):
    def test_getStateSet(self):
        """
        Node.sensitive should be False for the gtk-demo app node
        """
        self.assertFalse(self.app.sensitive)

    def test_setStateSet(self):
        """
        Node.stateSet should be read-only
        """
        # FIXME should be AttributeError?
        self.assertRaises(RuntimeError, self.app.__setattr__, "states", pyatspi.StateSet())

    # 'relations' (read-only list of atspi.Relation instances):
    def test_get_relations(self):
        # FIXME once relations are used for something other than labels
        pass

    # 'labelee' (read-only list of Node instances):
    def test_get_labelee(self):
        """
        Entry1/2's labelee should be a text widget
        """
        try:
            self.runDemo('Dialog and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs', roleName='frame', retry=False)
        except dogtail.tree.SearchError:
            self.runDemo('Dialogs and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs and Message Boxes', roleName='frame', retry=False)
        label = wnd.child(roleName='label', name='Entry 1')
        self.assertEqual(label.labelee.roleName, 'text')
        self.assertEqual(label.labellee.roleName, 'text')

    def test_set_labelee(self):
        """
        Node.labelee should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "labellee", None)

    def test_get_labeler(self):
        try:
            self.runDemo('Dialog and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs', roleName='frame', retry=False)
        except dogtail.tree.SearchError:
            self.runDemo('Dialogs and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs and Message Boxes', roleName='frame', retry=False)
        text = wnd.findChildren(dogtail.predicate.GenericPredicate(roleName='text'))[1]
        self.assertEqual(text.labeler.name, 'Entry 1')
        self.assertEqual(text.labeller.name, 'Entry 1')

    def test_set_labeller(self):
        """
        Node.labeller should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "labeller", None)

    # 'sensitive' (read-only boolean):
    def test_get_sensitive(self):
        """
        Node.sensitive should not be set for the gtk-demo app.
        It should be set for the window within the app.
        """
        self.assertFalse(self.app.sensitive)
        self.assertTrue(self.app.children[0].sensitive)

    def test_set_sensitive(self):
        """
        Node.sensitive should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "sensitive", True)

    # 'showing' (read-only boolean):
    def test_get_showing(self):
        """
        Node.showing should not be set for the gtk-demo.  It should be set for the window within the app
        """
        self.assertFalse(self.app.showing)
        self.assertTrue(self.app.children[0].showing)

    def test_set_showing(self):
        """
        Node.showing should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "showing", True)

    # 'visible' (read-only boolean):
    def test_get_visible(self):
        """
        Node.visible reflects a node property if is should be render or not
        """
        self.assertTrue(self.app.child(roleName='frame').visible)
        self.assertFalse(self.app.child(roleName='table column header').visible)

    def test_set_visible(self):
        """
        Node.visible should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "visible", False)

    # 'actions' (read-only list of Action instances):
    def test_get_actions(self):
        """
        Node.actions should be an empty list for the app node
        """
        self.assertEqual(len(self.app.actions), 0)

    def test_set_actions(self):
        """
        Node.actions should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "actions", {})

    # 'extents' (readonly tuple):
    def test_get_extents(self):
        """
        Node.extents should be a 4-tuple for a window, with non-zero size
        """
        (x, y, w, h) = self.app.children[0].extents
        self.assertTrue(w > 0)
        self.assertTrue(h > 0)

    def test_get_extens_wrong(self):
        """
        Some accessible do not specify size nor position
        """
        self.assertIsNone(self.app.extents)

    def test_set_extents(self):
        """
        Node.extents should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "extents", (0, 0, 640, 480))

    # 'position' (readonly tuple):
    def test_get_position(self):
        """
        Node.position should be a 2-tuple for a window
        """
        (x, y) = self.app.children[0].position
        self.assertTrue(isinstance(x, int))
        self.assertTrue(isinstance(y, int))

    def test_get_position_not_implemented(self):
        """
        Some accessible do not specify position (nor size)
        """
        with self.assertRaises(NotImplementedError):
            self.app.position

    def test_set_position(self):
        """
        Node.position should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "position", (0, 0))

    # 'size' (readonly tuple):
    def test_get_size(self):
        """
        Node.size should be a 2-tuple for a window, with non-zero values
        """
        (w, h) = self.app.children[0].size
        self.assertTrue(w > 0)
        self.assertTrue(h > 0)

    def test_get_size_not_implemented(self):
        """
        Some accessible do not specify size (nor position)
        """
        with self.assertRaises(NotImplementedError):
            self.app.size

    def test_set_size(self):
        """
        Node.size should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "size", (640, 480))

    # 'toolkitName' (readonly string):
    def test_get_toolkit(self):
        """
        Node.toolkitName is a read-only string
        """
        self.assertEqual(self.app.toolkitName, "gtk")

    def test_set_toolkit(self):
        """
        Node.toolkit should be read-only
        """
        self.assertRaises(AttributeError, self.app.__setattr__, "toolkitName", "gtk")

    # 'ID'
    def test_get_ID(self):
        """
        Node.id should be numeric
        """
        self.assertEqual(type(self.app.id), type(42))

    def test_set_ID(self):
        """
        Node.id should be read-only
        """
        self.assertRaises(AttributeError, setattr, self.app, "id", 42)

    def test_checked(self):
        self.runDemo("Application Class")
        wnd = dogtail.tree.root.application('gtk3-demo-application')
        wnd.menu("Preferences").click()
        checkbox = wnd.menu("Preferences").menuItem("Bold")
        status = checkbox.checked
        checkbox.click()
        self.assertEqual(checkbox.checked, not status)
        self.assertEqual(checkbox.isChecked, not status)

    def test_dead(self):
        self.assertFalse(self.app.dead)
        import os
        import signal
        os.kill(self.pid, signal.SIGKILL)
        dogtail.utils.doDelay(5)
        self.assertTrue(self.app.dead)

    def test_dead_empty(self):
        node = dogtail.tree.Node()
        self.assertTrue(node.dead)

    # https://bugzilla.gnome.org/show_bug.cgi?id=710730
    # GError: Method "Contains" with signature "iin" on interface "org.a11y.atspi.Component" doesn't exist
    def test_contains(self):
        child = self.app.children[0]
        position = child.position
        self.assertTrue(child.contains(position[0]+1, position[1]+1))

    def test_childAtPoint(self):
        top = self.app.children[0]
        bottom = top.children[1].children[1]
        position = bottom.position
        actual_acc = top.getChildAtPoint(position[0], position[1])
        self.assertEqual(actual_acc, bottom)

    def test_click(self):
        try:
            self.runDemo('Dialog and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs', roleName='frame', retry=False)
        except dogtail.tree.SearchError:
            self.runDemo('Dialogs and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs and Message Boxes', roleName='frame', retry=False)
        wnd.button('Interactive Dialog').click()
        self.assertTrue(self.app.dialog("Interactive Dialog").showing)

    def test_doubleClick(self):
        builder = self.app.child("Builder")
        self.assertEqual(len([x for x in self.app.children if x.name in ['GtkBuilder demo', 'Builder']]), 0)
        builder.doubleClick()
        self.assertEqual(len([x for x in self.app.children if x.name in ['GtkBuilder demo', 'Builder']]), 2)

    def test_point(self):
        self.runDemo('Application Class')
        wnd = dogtail.tree.root.application('gtk3-demo-application')
        wnd.menu("Preferences").click()
        color = wnd.menu("Preferences").menu("Color")
        red = wnd.menu("Preferences").menu("Color").menuItem("Red")
        self.assertFalse(red.showing)
        color.point()
        self.assertTrue(red.showing)

    def test_point_delay_explicit(self):
        self.runDemo('Application Class')
        wnd = dogtail.tree.root.application('gtk3-demo-application')
        wnd.menu("Preferences").click()
        color = wnd.menu("Preferences").menu("Color")
        red = wnd.menu("Preferences").menu("Color").menuItem("Red")
        self.assertFalse(red.showing)
        start = time.time()
        color.point(2.0)
        end = time.time()
        self.assertTrue(red.showing)
        self.assertTrue(end - start >= 2.0)

    def test_typeText_nonfucable(self):
        """
        Node.typeText on non-focusable node which is not input-friendly should end in an error
        """
        with self.assertRaises(NotImplementedError):
            self.app.typeText('Something')


class TestSelection(GtkDemoTest):

    def test_tabs(self):
        """
        Tabs in the gtk-demo should be selectable, and be queryable for
        "isSelected", and the results should change as they are selected.
        """
        # Use the Info/Source tabs of gtk-demo:
        info = self.app.child('Info')
        source = self.app.child('Source')

        # Check initial state:
        self.assertTrue(info.isSelected)
        self.assertFalse(source.isSelected)

        # Select other tab:
        source.select()

        # Check new state:
        self.assertFalse(info.isSelected)
        self.assertTrue(source.isSelected)

    def test_iconView(self):
        self.app.child(roleName='tree table').typeText("Icon View")
        self.app.child(roleName='tree table').child("Icon View").click()
        self.app.child(roleName='tree table').child("Icon View").typeText("+")
        self.runDemo('Icon View Basics')
        try:
            wnd = self.app.child('GtkIconView demo', roleName='frame', recursive=False, retry=False)
        except dogtail.tree.SearchError:
            wnd = self.app.child('Icon View Basics', roleName='frame', recursive=False, retry=False)

        pane = wnd.child(roleName="layered pane")
        icons = pane.children

        pane.selectAll()
        self.assertNotIn(False, [x.selected for x in icons])
        pane.select_all()
        self.assertNotIn(False, [x.selected for x in icons])
        pane.select_child(0)
        self.assertTrue(icons[0].selected)
        self.assertNotIn(False, [x.selected for x in icons])


class TestValue(GtkDemoTest):

    def test_get_value(self):
        """
        The scrollbar starts out at position zero.
        """
        sb = self.app.child(roleName='scroll bar')
        self.assertEqual(sb.value, 0)

    def test_set_value(self):
        """
        Ensure that we can set the value of the scrollbar.
        """
        sb = self.app.findChildren(dogtail.predicate.GenericPredicate(roleName='scroll bar'))[1]
        sb.value = 100
        self.assertEqual(sb.value, 100)

    def test_min_value(self):
        """
        Ensure that the minimum value for the scrollbar is correct.
        """
        sb = self.app.findChildren(dogtail.predicate.GenericPredicate(roleName='scroll bar'))[1]
        self.assertEqual(sb.minValue, 0)

    def test_max_value(self):
        sb = self.app.findChildren(dogtail.predicate.GenericPredicate(roleName='scroll bar'))[1]
        self.assertTrue(sb.maxValue > 250)

    def test_min_value_increment(self):
        """
        Ensure that the minimum value increment of the scrollbar is an int.
        """
        sb = self.app.findChildren(dogtail.predicate.GenericPredicate(roleName='scroll bar'))[1]
        self.assertEqual(sb.minValueIncrement, sb.minValueIncrement)


class TestSearching(GtkDemoTest):

    def test_findChildren(self):
        """
        Ensure that there are the correct number of table cells in the list
        of demos.
        """
        pred = dogtail.predicate.GenericPredicate(roleName='table cell')
        tableCells = self.app.findChildren(pred)

        def get_table_cells_recursively(node):
            counter = 0
            for child in node.children:
                if child.roleName == 'table cell':
                    counter += 1
                counter += get_table_cells_recursively(child)
            return counter

        counter = get_table_cells_recursively(self.app)
        self.assertEqual(len(tableCells), counter)

    def test_findChild_lambda(self):
        """
        Ensure that the lambda usage works as expected with Node.findChild
        """
        try:
            self.app.findChild(lambda x: x.roleName == 'page tab list')
        except dogtail.tree.SearchError:
            self.fail("Got a SearchError trying to find the page tab list")

    def test_findChildren2(self):
        """
        Ensure that there are two tabs in the second page tab list.
        """
        pred = dogtail.predicate.GenericPredicate(roleName='page tab list')
        pageTabList = self.app.findChildren(pred)
        pred = dogtail.predicate.GenericPredicate(roleName='page tab')
        # only one page tab list present since gtk3-demo 3.14
        pageTabs = pageTabList[0].findChildren(pred)
        self.assertEqual(len(pageTabs), 5)

    def test_findChildren2_lambda(self):
        """
        Ensure that there ic correct number of tabs in the first page tab list, use lambdas
        """
        pageTabList = self.app.findChildren(lambda x: x.roleName == 'page tab list')
        pageTabs = pageTabList[0].findChildren(lambda x: x.roleName == 'page tab')
        self.assertEqual(len(pageTabs), 5)

    def test_findChildren_lambdas(self):
        """
        Ensure that the lambda usage works as expected in Node.findChildren
        """
        try:
            self.runDemo('Dialog and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs', roleName='frame', retry=False)
        except dogtail.tree.SearchError:
            self.runDemo('Dialogs and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs and Message Boxes', roleName='frame', retry=False)
        texts = wnd.findChildren(lambda x: x.roleName == 'text', isLambda=True)
        self.assertEqual(len(texts), 2)
        self.assertEqual(texts[0].roleName, 'text')
        self.assertEqual(texts[1].roleName, 'text')
        texts1 = wnd.findChildren(lambda x: x.roleName == 'text' and x.labeler.name == 'Entry 1', isLambda=True)
        self.assertEqual(len(texts1), 1)
        self.assertEqual(texts1[0].roleName, 'text')
        self.assertEqual(texts1[0].labeler.name, 'Entry 1')
        texts2 = wnd.findChildren(lambda x: x.roleName == 'text' and x.showing, isLambda=True)
        self.assertEqual(len(texts2), 2)
        self.assertEqual(texts2[0].roleName, 'text')
        self.assertTrue(texts2[0].showing)
        self.assertEqual(texts2[1].roleName, 'text')
        self.assertTrue(texts2[1].showing)

    def test_findAncestor(self):
        pred = dogtail.predicate.GenericPredicate(roleName='table cell')
        child = self.app.child("Builder")
        parent = child.findAncestor(pred)
        self.assertIn(child, parent.children)
        pred = dogtail.predicate.GenericPredicate(roleName='frame')
        parent = child.findAncestor(pred)
        self.assertIn(parent, self.app.children)
        # No ancestor found
        self.assertIsNone(parent.findAncestor(pred))

    def test_isChild(self):
        parent = self.app.child(roleName='tree table')
        self.assertTrue(parent.isChild("Builder"))

    def test_getUserVisibleStrings(self):
        child = self.app.child("Builder")
        self.assertEqual(child.getUserVisibleStrings(), ['Builder'])

    def test_satisfies(self):
        pred = dogtail.predicate.GenericPredicate(roleName='table cell')
        builder = self.app.child("Builder")
        self.assertTrue(builder.satisfies(pred))

    def test_absoluteSearchPath(self):
        self.assertEqual(
            str(self.app.getAbsoluteSearchPath()),
            '{/("gtk3-demo" application,False)}')
        builder = self.app.child("Builder")
        self.assertEqual(
            str(builder.getAbsoluteSearchPath()),
            '{/("gtk3-demo" application,False)/("Application Class" window,False)/(child with name="Builder" '
            'roleName=\'table cell\',True)}')

    def test_compare_equal_search_paths(self):
        builder = self.app.child("Builder")
        builder_sp = builder.getAbsoluteSearchPath()
        self.assertTrue(builder_sp == builder_sp)

    def test_compare_unequal_search_paths_different_length(self):
        builder = self.app.child("Builder")
        builder_sp = builder.getAbsoluteSearchPath()
        app_sp = self.app.getAbsoluteSearchPath()
        self.assertFalse(builder_sp == app_sp)

    def test_compare_unequal_search_paths_same_length(self):
        builder = self.app.child("Builder")
        assistant = self.app.child("Assistant")
        builder_sp = builder.getAbsoluteSearchPath()
        assistant_sp = assistant.getAbsoluteSearchPath()
        self.assertFalse(builder_sp == assistant_sp)

    def test_get_search_path_length(self):
        builder = self.app.child("Builder")
        builder_sp = builder.getAbsoluteSearchPath()
        self.assertEqual(builder_sp.length(), 3)

    def test_iterate_search_path(self):
        builder = self.app.child("Builder")
        builder_sp = builder.getAbsoluteSearchPath()
        self.assertEqual(
            [x[0].makeScriptVariableName() for x in builder_sp],
            ['gtk3DemoApp', 'applicationClassWin', 'builderNode'])

    def test_make_script_method_call_from_search_path(self):
        builder = self.app.child("Builder")
        builder_sp = builder.getAbsoluteSearchPath()
        # FIXME: dot in the beginning
        self.assertEqual(
            builder_sp.makeScriptMethodCall(),
            '.application("gtk3-demo").window("Application Class").child( name="Builder" roleName=\'table cell\')')

    def test_get_relative_search_path_for_path(self):
        builder = self.app.child("Builder")
        builder_sp = builder.getAbsoluteSearchPath()
        frame_sp = self.app.window("Application Class").getAbsoluteSearchPath()
        # FIXME: Should be "child( name="Widget (double click for demo)" roleName=\'page tab\').child( name="Builder""
        #                  " roleName=\'table cell\')"
        self.assertIsNone(builder_sp.getRelativePath(frame_sp))

    def test_get_prefix_for_search_path(self):
        builder = self.app.child("Builder")
        builder_sp = builder.getAbsoluteSearchPath()
        self.assertEqual(
            str(builder_sp.getPrefix(1)),
            '{/("gtk3-demo" application,False)}')

    def test_get_predicate(self):
        builder = self.app.child("Builder")
        builder_sp = builder.getAbsoluteSearchPath()
        pred = builder_sp.getPredicate(0)
        self.assertEqual(type(pred), dogtail.predicate.IsAnApplicationNamed)
        self.assertEqual(str(pred.appName), '"gtk3-demo"')

    def test_getRelativeSearch_app(self):
        relpath = self.app.getRelativeSearch()
        self.assertEqual(str(relpath[0]), '[desktop frame | main]')
        self.assertEqual(relpath[1].name.untranslatedString, 'gtk3-demo')
        self.assertFalse(relpath[2])

    def test_getRelativeSearch_widget(self):
        builder = self.app.child("Builder")
        relpath = builder.getRelativeSearch()
        self.assertEqual(str(relpath[0]), '[frame | Application Class]')
        self.assertEqual(relpath[1].describeSearchResult(), 'child with name="Builder" roleName=\'table cell\'')
        self.assertTrue(relpath[2])

    def test_findChildren_non_recursive(self):
        dogtail.rawinput.keyCombo("<End>")
        self.app.child(roleName='tree table').child("Tree View").click()
        self.app.child(roleName='tree table').child("Tree View").typeText("+")
        self.runDemo('Tree Store')
        try:
            wnd = self.app.child('Card planning sheet', roleName='frame', retry=False, recursive=False)
        except dogtail.tree.SearchError:
            wnd = self.app.child('Tree Store', roleName='frame', retry=False, recursive=False)
        table = wnd.child(roleName='tree table')
        pred = dogtail.predicate.GenericPredicate(roleName='table cell')
        dogtail.config.config.childrenLimit = 10000
        cells = table.findChildren(pred, recursive=False)
        direct_cells = [cell for cell in table.children if cell.roleName == 'table cell']
        self.assertEqual(len(cells), len(direct_cells))

    def test_find_by_shortcut(self):
        self.runDemo('Builder')
        try:
            wnd = self.app.child('GtkBuilder demo', roleName='frame', recursive=False, retry=False)
        except dogtail.tree.SearchError:
            wnd = self.app.child('Builder', roleName='frame', recursive=False, retry=False)

        self.assertIsNotNone(wnd.menu("File"))
        self.assertIsNotNone(wnd.menu("File").menuItem("New"))
        self.assertIsNotNone(wnd.button("Save"))
        self.assertIsNotNone(wnd.childNamed("File"))
        self.assertIsNotNone(self.app.tab("Info"))

    def test_find_by_shortcut2(self):
        try:
            self.runDemo('Dialog and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs', roleName='frame', retry=False)
        except dogtail.tree.SearchError:
            self.runDemo('Dialogs and Message Boxes', retry=False)
            wnd = self.app.child('Dialogs and Message Boxes', roleName='frame', retry=False)
        self.assertIsNotNone(wnd.childLabelled("Entry 1"))
        self.assertIsNotNone(wnd.button("Message Dialog"))


# A painful point of collision between strings in python2 and python3!
@unittest.skipIf(os.system('ls /usr/bin/gedit') != 0, "Skipping, need gedit")
class TestUnicodeNames(unittest.TestCase):

    def setUp(self):
        import dogtail.config
        dogtail.config.config.logDebugToStdOut = True
        dogtail.config.config.logDebugToFile = True
        dogtail.config.config.logDebugToStdOut = True
        dogtail.config.config.debugSearching = True
        dogtail.config.config.searchCutoffCount = 3
        import dogtail.utils
        self.pid = dogtail.utils.run('gedit')
        self.app = dogtail.tree.root.application('gedit')

    def test_unicode_char_in_name(self):
        self.app.child('Open', roleName='toggle button').click()
        unicode_button = None
        unicode_button = self.app.child(name=u'Other Documents…', roleName='push button')
        assert unicode_button is not None

    def test_unicode_char_in_name_click(self):
        self.app.child('Open', roleName='toggle button').click()
        unicode_button = self.app.child(name=u'Other Documents…', roleName='push button')
        unicode_button.click()
        dialog = None
        try:
            dialog = self.app.child(name=u'Open', roleName='file chooser')
        except dogtail.tree.SearchError:
            self.fail()
        assert dialog is not None

    def test_unicode_logging_nocrash(self):
        try:
            self.app.child(name='…Other stuff…', roleName='push button')
            self.fail()
        except dogtail.tree.SearchError:
            pass

    def tearDown(self):
        import signal
        os.kill(self.pid, signal.SIGKILL)
        os.system('killall gedit > /dev/null 2>&1')
        # Sleep just enough to let the app actually die.
        # AT-SPI doesn't like being hammered too fast.
        time.sleep(0.5)

class TestDump(GtkDemoTest):

    def test_dump_to_stdout(self):
        child = self.app.child('Source')
        output = trap_stdout(child.dump)
        self.assertEqual(
            output,
            """[page tab | Source]
 [scroll pane | ]
  [text | ]
  [scroll bar | ]
  [scroll bar | ]""")

    def test_dump_with_actions(self):
        child = self.app.child('Builder', roleName='table cell')
        output = trap_stdout(child.dump)
        self.assertEqual(
            output,
            """[table cell | Builder]
 [action | activate |  ]
 [action | edit |  ]
 [action | expand or contract |  ]""")



#!/usr/bin/env python
"""
Dogtail script for generating unhelpful help file.

Scours the UI of a program, and generates a parody of a help file for that
program.

The resulting DocBook file is simply a reading back of the program's UI to you.

The idea is to highlight a gripe of mine (and many other people):
documentation should be more than this!

This script is licensed under the GPL.
"""
__author__ = 'David Malcolm <dmalcolm@redhat.com>'

from dogtail.tree import *
from dogtail.utils import *
import sys, cgi

def writePCDataElement(name, content):
    print(('<%s>%s</%s>' % (name, content, name)))

def generateUnhelpfulHelp(appName):
    try:
        app = root.application(appName)
    except SearchError:
        run(appName)
    print('<?xml version="1.0"?>')
    print('<!DOCTYPE article PUBLIC "-//OASIS//DTD DocBook XML V4.1.2//EN" "http://www.oasis-open.org/docbook/xml/4.1.2/docbookx.dtd">')
    print('<article>')

    taggedAppName = '<application>%s</application>'%cgi.escape(appName)

    # Artile info:
    print('\t<articleinfo>')
    writePCDataElement("title", "The Totally Definitive No-Nonsense Unhelpful Complete Guide to %s for Mannequins in 40 Days - Unleashed!"%taggedAppName)
    print('\t</articleinfo>')

    # Guts of the article:
    print('\t<sect1>')
    writePCDataElement("title", "Introduction")
    writePCDataElement("caution", "<para>Do not take these instructions seriously.  They are a parody of unhelpful help files found on many computer systems, and were autogenerated by <application>no-help-at-all</application></para>")
    writePCDataElement("para", "You can start %s by opening a terminal and typing the <command>%s</command> command."%(taggedAppName, cgi.escape(appName)))
    print('\t</sect1>')

    menus = app.findChildren(predicate.GenericPredicate(roleName="menu"))
    for menu in menus:
        print('\t<sect1>')
        writePCDataElement("title", "The <guimenu>%s</guimenu> menu"%cgi.escape(menu.name))
        writePCDataElement("para", "Use the <guimenu>%s</guimenu> menu to work with %ss"%(cgi.escape(menu.name), cgi.escape(menu.name.lower())))
        items = menu.findChildren(predicate.GenericPredicate(roleName="menu item"), recursive=False)
        if items!=None:
            for item in items:
                verb = item.name
                noun = menu.name
                print('\t\t<sect2>')
                writePCDataElement("title", "%sing a %s"%(cgi.escape(verb.title()), cgi.escape(noun.lower())))
                writePCDataElement("para", "To %s a %s, choose <guimenu>%s</guimenu> &gt; <guimenuitem>%s</guimenuitem>"%(cgi.escape(verb.lower()), cgi.escape(noun.lower()), cgi.escape(menu.name), cgi.escape(item.name)))
                print('\t\t</sect2>')
        print('\t</sect1>')

    print('</article>')

if __name__=='__main__':
    try:
        generateUnhelpfulHelp(sys.argv[1])
    except IndexError:
        print("####################################")
        print("please supply an application name on the cmdline")
        print("####################################")



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
from dogtail import errors
from dogtail import predicate
from dogtail import rawinput
from dogtail import tree
from dogtail.config import config
from dogtail.utils import Lock
import os

"""
Dogtail's procedural UI
All the classes here are intended to be single-instance, except for Action.
"""
__author__ = 'Zack Cerza <zcerza@redhat.com>'
#
#
# WARNING: Here There Be Dragons (TM)                                        #
#                                                                            #
# If you don't understand how to use this API, you almost certainly don't    #
# want to read the code first. We make use of some very non-intuitive        #
# features of Python in order to make the API very simplistic. Therefore,    #
# you should probably only read this code if you're already familiar with    #
# some of Python's advanced features. You have been warned. ;)               #
#
#


class FocusError(Exception):
    pass


def focusFailed(pred):
    errors.warn('The requested widget could not be focused: %s' % pred.debugName)

ENOARGS = "At least one argument is needed"


class FocusBase(object):
    """
    The base for every class in the module. Does nothing special, really.
    """
    node = None

    def __getattr__(self, name):
        # Fold all the Node's AT-SPI properties into the Focus object.
        try:
            return getattr(self.node, name)
        except AttributeError:
            raise AttributeError(name)

    def __setattr__(self, name, value):
        # Fold all the Node's AT-SPI properties into the Focus object.
        if name == 'node':
            setattr(self.__class__, name, value)
        else:
            try:
                setattr(self.node, name, value)
            except AttributeError:
                raise AttributeError(name)


class FocusApplication(FocusBase):
    """
    Keeps track of which application is currently focused.
    """
    desktop = tree.root

    def __call__(self, name):
        """
        Search for an application that matches and refocus on the given name.
        """
        try:
            pred = predicate.IsAnApplicationNamed(name)
            app = self.desktop.findChild(pred, recursive=False, retry=False)
        except tree.SearchError:
            if config.fatalErrors:
                raise FocusError(name)
            else:
                focusFailed(pred)
                return False
        if app:
            FocusApplication.node = app
            FocusDialog.node = None
            FocusWindow.node = None
            FocusWidget.node = None
        return True


class FocusDesktop(FocusBase):
    """
    This isn't used yet, and may never be used.
    """
    pass


class FocusWindow(FocusBase):
    """
    Keeps track of which window is currently focused.
    """

    def __call__(self, name):
        """
        Search for a dialog that matches the given name and refocus on it.
        """
        result = None
        pred = predicate.IsAWindowNamed(name)
        try:
            result = FocusApplication.node.findChild(pred, requireResult=False, recursive=False)
        except AttributeError:
            pass
        if result:
            FocusWindow.node = result
            FocusDialog.node = None
            FocusWidget.node = None
        else:
            if config.fatalErrors:
                raise FocusError(pred.debugName)
            else:
                focusFailed(pred)
                return False
        return True


class FocusDialog(FocusBase):
    """
    Keeps track of which dialog is currently focused.
    """

    def __call__(self, name):
        """
        Search for a dialog that matches the given name and refocus on it.
        """
        result = None
        pred = predicate.IsADialogNamed(name)
        try:
            result = FocusApplication.node.findChild(pred, requireResult=False, recursive=False)
        except AttributeError:
            pass
        if result:
            FocusDialog.node = result
            FocusWidget.node = None
        else:
            if config.fatalErrors:
                raise FocusError(pred.debugName)
            else:
                focusFailed(pred)
                return False
        return True


class FocusWidget(FocusBase):
    """
    Keeps track of which widget is currently focused.
    """

    def findByPredicate(self, pred):
        result = None
        try:
            result = FocusWidget.node.findChild(pred, requireResult=False, retry=False)
        except AttributeError:
            pass
        if result:
            FocusWidget.node = result
        else:
            try:
                result = FocusDialog.node.findChild(pred, requireResult=False, retry=False)
            except AttributeError:
                pass
        if result:
            FocusWidget.node = result
        else:
            try:
                result = FocusWindow.node.findChild(pred, requireResult=False, retry=False)
            except AttributeError:
                pass
        if result:
            FocusWidget.node = result
        else:
            try:
                result = FocusApplication.node.findChild(pred, requireResult=False, retry=False)
                if result:
                    FocusWidget.node = result
            except AttributeError:
                if config.fatalErrors:
                    raise FocusError(pred)
                else:
                    focusFailed(pred)
                    return False

        if result is None:
            FocusWidget.node = result
            if config.fatalErrors:
                raise FocusError(pred.debugName)
            else:
                focusFailed(pred)
                return False
        return True

    def __call__(self, name='', roleName='', description=''):
        """
        If name, roleName or description are specified, search for a widget that matches and refocus on it.
        """
        if not name and not roleName and not description:
            raise TypeError(ENOARGS)

        # search for a widget.
        pred = predicate.GenericPredicate(name=name, roleName=roleName, description=description)
        return self.findByPredicate(pred)


class Focus(FocusBase):
    """
    The container class for the focused application, dialog and widget.
    """

    def __getattr__(self, name):
        raise AttributeError(name)

    def __setattr__(self, name, value):
        if name in ('application', 'dialog', 'widget', 'window'):
            self.__dict__[name] = value
        else:
            raise AttributeError(name)

    desktop = tree.root
    application = FocusApplication()
    app = application  # shortcut :)
    dialog = FocusDialog()
    window = FocusWindow()
    frame = window
    widget = FocusWidget()

    def button(self, name):
        """
        A shortcut to self.widget.findByPredicate(predicate.IsAButtonNamed(name))
        """
        return self.widget.findByPredicate(predicate.IsAButtonNamed(name))

    def icon(self, name):
        """
        A shortcut to self.widget(name, roleName = 'icon')
        """
        return self.widget(name=name, roleName='icon')

    def menu(self, name):
        """
        A shortcut to self.widget.findByPredicate(predicate.IsAMenuNamed(name))
        """
        return self.widget.findByPredicate(predicate.IsAMenuNamed(name))

    def menuItem(self, name):
        """
        A shortcut to self.widget.findByPredicate(predicate.IsAMenuItemNamed(name))
        """
        return self.widget.findByPredicate(predicate.IsAMenuItemNamed(name))

    def table(self, name=''):
        """
        A shortcut to self.widget(name, roleName 'table')
        """
        return self.widget(name=name, roleName='table')

    def tableCell(self, name=''):
        """
        A shortcut to self.widget(name, roleName 'table cell')
        """
        return self.widget(name=name, roleName='table cell')

    def text(self, name=''):
        """
        A shortcut to self.widget.findByPredicate(IsATextEntryNamed(name))
        """
        return self.widget.findByPredicate(predicate.IsATextEntryNamed(name))


class Action(FocusWidget):
    """
    Aids in executing AT-SPI actions, refocusing the widget if necessary.
    """

    def __init__(self, action):
        """
        action is a string with the same name as the AT-SPI action you wish to execute using this class.
        """
        self.action = action

    def __call__(self, name='', roleName='', description='', delay=config.actionDelay):
        """
        If name, roleName or description are specified, first search for a widget that matches and refocus on it.
        Then execute the action.
        """
        if name or roleName or description:
            FocusWidget.__call__(self, name=name, roleName=roleName, description=description)
        self.node.doActionNamed(self.action)

    def __getattr__(self, attr):
        return getattr(FocusWidget.node, attr)

    def __setattr__(self, attr, value):
        if attr == 'action':
            self.__dict__[attr] = value
        else:
            setattr(FocusWidget, attr, value)

    def button(self, name):
        """
        A shortcut to self(name, roleName = 'push button')
        """
        self.__call__(name=name, roleName='push button')

    def menu(self, name):
        """
        A shortcut to self(name, roleName = 'menu')
        """
        self.__call__(name=name, roleName='menu')

    def menuItem(self, name):
        """
        A shortcut to self(name, roleName = 'menu item')
        """
        self.__call__(name=name, roleName='menu item')

    def table(self, name=''):
        """
        A shortcut to self(name, roleName 'table')
        """
        self.__call__(name=name, roleName='table')

    def tableCell(self, name=''):
        """
        A shortcut to self(name, roleName 'table cell')
        """
        self.__call__(name=name, roleName='table cell')

    def text(self, name=''):
        """
        A shortcut to self(name, roleName = 'text')
        """
        self.__call__(name=name, roleName='text')


class Click(Action):
    """
    A special case of Action, Click will eventually handle raw mouse events.
    """
    primary = 1
    middle = 2
    secondary = 3

    def __init__(self):
        Action.__init__(self, 'click')

    def __call__(self, name='', roleName='', description='', raw=True, button=primary, delay=config.actionDelay):
        """
        By default, execute a raw mouse event.
        If raw is False or if button evaluates to False, just pass the rest of
        the arguments to Action.
        """
        if name or roleName or description:
            FocusWidget.__call__(self, name=name, roleName=roleName, description=description)
        if raw and button:
            # We're doing a raw mouse click
            Click.node.click(button)
        else:
            Action.__call__(self, name=name, roleName=roleName, description=description, delay=delay)


class Select(Action):
    """
    Aids in selecting and deselecting widgets, i.e. page tabs
    """
    select = 'select'
    deselect = 'deselect'

    def __init__(self, action):
        """
        action must be 'select' or 'deselect'.
        """
        if action not in (self.select, self.deselect):
            raise ValueError(action)
        Action.__init__(self, action)

    def __call__(self, name='', roleName='', description='', delay=config.actionDelay):
        """
        If name, roleName or description are specified, first search for a widget that matches and refocus on it.
        Then execute the action.
        """
        if name or roleName or description:
            FocusWidget.__call__(self, name=name, roleName=roleName, description=description)
        func = getattr(self.node, self.action)
        func()


def type(text):
    if focus.widget.node:
        focus.widget.node.typeText(text)
    else:
        rawinput.typeText(text)


def keyCombo(combo):
    if focus.widget.node:
        focus.widget.node.keyCombo(combo)
    else:
        rawinput.keyCombo(combo)


def run(application, arguments='', appName=''):
    from dogtail.utils import run as utilsRun
    pid = utilsRun(application + ' ' + arguments, appName=appName)
    focus.application(application)
    return pid

# tell sniff not to use auto-refresh while script using this module is running
# may have already been locked by dogtail.tree
if not os.path.exists('/tmp/sniff_refresh.lock'):  # pragma: no cover
    # this lock will automatically unlock on script exit.
    sniff_lock = Lock(lockname='sniff_refresh.lock', randomize=False, unlockOnExit=True)
    try:
        sniff_lock.lock()
    except OSError:
        pass  # lock was already present from other script instance or leftover from killed instance

focus = Focus()
click = Click()
activate = Action('activate')
openItem = Action('open')
menu = Action('menu')
select = Select(Select.select)
deselect = Select(Select.deselect)



#!/usr/bin/env python
from __future__ import absolute_import, division, print_function, unicode_literals
from dogtail.procedural import focus, keyCombo, deselect, activate, select, click, tree, FocusError, run, config, type
from dogtail.tree import SearchError, ActionNotSupported, NotSensitiveError
from gtkdemotest import GtkDemoTest, trap_stdout
from time import sleep
import pyatspi

"""
Unit tests for the dogtail.procedural API
"""
__author__ = "Zack Cerza <zcerza@redhat.com>"

config.logDebugToFile = False
config.logDebugToStdOut = True


class GtkDemoTest(GtkDemoTest):

    def setUp(self):
        self.pid = run('gtk3-demo')
        self.app = focus.application.node


class TestFocusApplication(GtkDemoTest):

    def test_focusing_bogus_name_without_fatal_error(self):
        config.fatalErrors = False
        output = trap_stdout(focus.application, "should not be found")
        self.assertTrue('The requested widget could not be focused: "should not be found" application' in output)

    def test_throw_exception_on_focusing_bogus_name(self):
        config.fatalErrors = True
        self.assertRaises(FocusError, focus.application, "should not be found")

    def test_focusing_basic(self):
        """
        Ensure that focus.application() sets focus.application.node properly
        """
        focus.application.node = None
        focus.application("gtk3-demo")
        self.assertEqual(focus.application.node, self.app)

    def test_throw_exception_on_get_no_such_attribute(self):
        with self.assertRaises(AttributeError):
            focus.no_such_attribute

    def test_throw_exception_on_get_no_such_attribute_when_node_doesnt_exist(self):
        focus.application.node = None
        with self.assertRaises(AttributeError):
            focus.no_such_attribute

    def test_throw_exception_on_set_no_such_attribute(self):
        with self.assertRaises(AttributeError):
            focus.no_such_attribute = 0


class TestFocusWindow(GtkDemoTest):

    def test_focusing_bogus_name_without_a_fatal_error(self):
        config.fatalErrors = False
        output = trap_stdout(focus.window, "should not be found")
        self.assertIsNone(focus.window.node)
        self.assertIn('The requested widget could not be focused: "should not be found" window', output)

    def test_throw_exception_on_focusing_bogus_name(self):
        config.fatalErrors = True
        self.assertRaises(FocusError, focus.window, "should not be found")


class TestFocusDialog(GtkDemoTest):

    def test_focusing_bogus_name_without_a_fatal_error(self):
        config.fatalErrors = False
        output = trap_stdout(focus.dialog, "should not be found")
        self.assertIsNone(focus.dialog.node)
        self.assertIn('The requested widget could not be focused: "should not be found" dialog', output)

    def test_throw_exception_on_focusing_bogus_name(self):
        config.fatalErrors = True
        self.assertRaises(FocusError, focus.dialog, "should not be found")


class TestFocusWidget(GtkDemoTest):

    def test_focusing_empty_name(self):
        self.assertRaises(TypeError, focus.widget)

    def test_focusing_bogus_name_without_a_fatal_error(self):
        config.fatalErrors = False
        output = trap_stdout(focus.widget, "should not be found")
        self.assertIsNone(focus.widget.node)
        self.assertIn('The requested widget could not be focused: child with name="should not be found"', output)

    def test_throw_exception_on_focusing_bogus_name(self):
        config.fatalErrors = True
        self.assertRaises(FocusError, focus.widget, "should not be found")

    def test_focusing_basic(self):
        """
        Ensure that focus.widget('foo') finds a node with name 'foo'
        """
        focus.widget("Application Class")
        self.assertEqual(focus.widget.name, "Application Class")


class TestFocus(GtkDemoTest):

    def test_initial_state(self):
        """
        Ensure that focus.widget, focus.dialog and focus.window are None initially.
        """
        self.assertIsNone(focus.widget.node)
        self.assertIsNone(focus.dialog.node)
        self.assertIsNone(focus.window.node)

    def test_focusing_app(self):
        """
        Ensure that focus.app() works
        """
        focus.app.node = None
        focus.app('gtk3-demo')
        self.assertEqual(focus.app.node, self.app)

    def test_focusing_app_via_application(self):
        """
        Ensure that focus.application() works
        """
        focus.app.node = None
        focus.application('gtk3-demo')
        self.assertEqual(focus.app.node, self.app)

    def test_focus_getting_bogus_attribute(self):
        self.assertRaises(AttributeError, getattr, focus, 'nosuchtype')

    def test_focus_setting_bogus_attribute(self):
        self.assertRaises(AttributeError, setattr, focus, 'nosuchtype', 'nothing')

    def test_focusing_roleName(self):
        """
        Ensure that focus.widget(roleName=...) works.
        """
        focus.widget(roleName='page tab')
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_PAGE_TAB)

    def test_focus_menu(self):
        self.runDemo('Builder')
        focus.menu('File')
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_MENU)

    def test_focus_menuItem(self):
        self.runDemo('Builder')
        click.menu('File')
        focus.menuItem('New')
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_MENU_ITEM)

    def test_focus_button(self):
        self.runDemo('Builder')
        focus.button('Open')
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_PUSH_BUTTON)

    def test_focus_table(self):
        self.runDemo('Builder')
        focus.table('')
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_TABLE)

    def test_focus_tableCell(self):
        self.runDemo('Builder')
        focus.tableCell('')
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_TABLE_CELL)

    def test_focus_text(self):
        self.runDemo('Assistant')
        focus.window('Page 1')
        focus.text('')
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_TEXT)

    def test_focus_icon(self):
        self.runDemo('Clipboard')
        try:
            wnd = self.app.child('Clipboard demo', roleName='frame', retry=False, recursive=False)
        except SearchError:
            wnd = self.app.child('Clipboard', roleName='frame', retry=False, recursive=False)
        focus.window(wnd.name)
        focus.icon('Warning')
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_ICON)


class TestKeyCombo(GtkDemoTest):

    def test_keyCombo(self):
        self.runDemo('Builder')
        keyCombo('<F7>')
        res = False
        try:
            res = focus.dialog('About Builder demo')
        except:
            try:
                res = focus.dialog('About GtkBuilder demo')
            except:
                pass
        self.assertTrue(res)

    def test_keyCombo_on_widget(self):
        self.runDemo('Builder')
        focus.button('Copy')
        keyCombo('<F7>')
        try:
            res = focus.dialog('About Builder demo')
        except:
            try:
                res = focus.dialog('About GtkBuilder demo')
            except:
                pass
        self.assertTrue(res)


class TestActions(GtkDemoTest):

    def test_click(self):
        click('Source')
        self.assertTrue(focus.widget.isSelected)

    def test_click_on_invisible_element(self):
        with self.assertRaises(ValueError):
            click("Spinner")

    def test_click_with_raw(self):
        click('Source', raw=True)
        self.assertTrue(focus.widget.isSelected)

    def test_select(self):
        select('Source')
        self.assertTrue(focus.widget.isSelected)

    def test_deselect(self):
        type('Icon View')
        click('Icon View')
        type('+')
        sleep(0.5)
        self.runDemo('Icon View Basics')
        try:
            wnd = self.app.child('GtkIconView demo', roleName='frame', recursive=False, retry=False)
        except SearchError:
            wnd = self.app.child('Icon View Basics', roleName='frame', recursive=False, retry=False)
        focus.window(wnd.name)

        focus.widget(roleName='icon')
        select()
        deselect()
        self.assertFalse(focus.widget.isSelected)

    def test_typing_on_widget(self):
        try:
            self.runDemo('Dialog and Message Boxes', retry=False)
            focus.window('Dialogs')
        except SearchError:
            self.runDemo('Dialogs and Message Boxes', retry=False)
            focus.window('Dialogs and Message Boxes')
        focus.widget(roleName='text')
        type("hello world")
        from time import sleep
        sleep(0.1)
        self.assertEqual(focus.widget.node.text, 'hello world')

    def test_custom_actions(self):
        activate("Combo Boxes")
        self.assertEqual(focus.widget.node.text, 'Combo Boxes')

    def test_blink_on_actions(self):
        config.blinkOnActions = True
        activate("Combo Boxes")
        self.assertEqual(focus.widget.node.text, 'Combo Boxes')

    def test_custom_actions_button(self):
        try:
            self.runDemo('Dialog and Message Boxes', retry=False)
            focus.window('Dialogs')
        except SearchError:
            self.runDemo('Dialogs and Message Boxes', retry=False)
            focus.window('Dialogs and Message Boxes')
        click.button('Interactive Dialog')
        self.assertTrue(focus.dialog("Interactive Dialog"))

    def test_custom_actions_menu(self):
        self.runDemo('Builder')
        try:
            wnd = self.app.child('GtkBuilder demo', roleName='frame', recursive=False, retry=False)
        except SearchError:
            wnd = self.app.child('Builder', roleName='frame', recursive=False, retry=False)
        focus.window(wnd.name)
        click.menu('File')
        click.menuItem('New')
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_MENU_ITEM)

    def test_custom_actions_text(self):
        self.runDemo('Builder')
        try:
            wnd = self.app.child('GtkBuilder demo', roleName='frame', recursive=False, retry=False)
        except SearchError:
            wnd = self.app.child('Builder', roleName='frame', recursive=False, retry=False)
        focus.window(wnd.name)
        click.text('')
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_TEXT)

    def test_custom_actions_text_with_debug(self):
        self.runDemo('Builder')
        try:
            wnd = self.app.child('GtkBuilder demo', roleName='frame', recursive=False, retry=False)
        except SearchError:
            wnd = self.app.child('Builder', roleName='frame', recursive=False, retry=False)
        focus.window(wnd.name)
        config.debugSearching = True
        output = trap_stdout(click.text, '')
        self.assertIn(
            "searching for descendent of [frame | %s]: child with roleName='text'" % wnd.name,
            output)

        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_TEXT)

    def test_custom_actions_table_cell(self):
        activate.tableCell("Combo Boxes")
        self.assertTrue(isinstance(focus.widget.node, tree.Node))
        self.assertEqual(focus.widget.node.role, pyatspi.ROLE_TABLE_CELL)

    def test_throws_action_not_supported(self):
        self.runDemo('Builder')
        try:
            wnd = self.app.child('GtkBuilder demo', roleName='frame', recursive=False, retry=False)
        except SearchError:
            wnd = self.app.child('Builder', roleName='frame', recursive=False, retry=False)
        focus.window(wnd.name)
        with self.assertRaises(ActionNotSupported) as cm:
            activate.text('')
        self.assertEqual(str(cm.exception), "Cannot do 'activate' action on [text | ]")

    def test_action_on_insensitive(self):
        self.runDemo("Assistant")
        wnd = self.app.child("Page 1", roleName='frame')
        child = wnd.child("Next")
        config.ensureSensitivity = True
        with self.assertRaises(NotSensitiveError):
            child.actions['click'].do()
        config.ensureSensitivity = False
        output = trap_stdout(child.actions['click'].do)
        self.assertEqual(
            output,
            """click on [push button | Next]
Warning: Cannot click [push button | Next]. It is not sensitive.""")



#!/usr/bin/env python
# Stress test: repeatedly open and close the filechooser dialog

from dogtail.tree import *
from dogtail.utils import run
from sys import exit

run('gedit')

gedit = root.application('gedit')

while True:
    try:
        gedit.child('Open').click()
    except SearchError: #toolbar not present?
        gedit.child('Open...').click()

    try:
        filechooser = gedit.child(name='Open Files', roleName='file chooser')
        filechooser.childNamed('Cancel').click()
    except SearchError:
        print('File chooser did not open')
        exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
import dogtail.tree
from gtkdemotest import trap_stdout
import unittest

"""
Unit tests for the dogtail.logging package
"""


class TestLogging(unittest.TestCase):

    def setUp(self):
        self.old_log_dir = dogtail.config.config.logDir

    def tearDown(self):
        dogtail.config.config.logDebugToFile = False
        dogtail.config.config.logDir = self.old_log_dir

    def test_entryStamp_is_not_empty(self):
        ts = dogtail.logging.TimeStamp()
        self.assertEqual(len(ts.entryStamp()) > 0, True)

    def test_correct_error_if_log_dir_does_not_exist(self):
        import shutil
        shutil.rmtree(dogtail.config.config.logDir)
        self.assertRaises(IOError, dogtail.logging.Logger, "log", file=True)

    def test_unique_name(self):
        logger1 = dogtail.logging.Logger("log", file=True)
        logger1.createFile()
        logger2 = dogtail.logging.Logger("log", file=True)
        logger2.createFile()
        logger3 = dogtail.logging.Logger("log", file=True)
        self.assertNotEqual(logger1.fileName, logger2.fileName)
        self.assertNotEqual(logger2.fileName, logger3.fileName)

    def test_no_new_line_to_file(self):
        dogtail.config.config.logDebugToFile = True
        logger = dogtail.logging.Logger("log", file=True, stdOut=False)
        logger.log("hello world", newline=False)
        with open(logger.fileName, 'r') as f:
            logger_text = f.read()
        self.assertTrue("hello world " in logger_text)

    def test_no_new_line_to_stdout(self):
        dogtail.config.config.logDebugToFile = False
        logger = dogtail.logging.Logger("log", file=False, stdOut=True)
        output = trap_stdout(
            logger.log, {'message': 'hello world', 'newline': False})
        self.assertEqual(output, "hello world")

    def test_no_new_line_to_both_file_and_stdout(self):
        dogtail.config.config.logDebugToFile = True
        logger = dogtail.logging.Logger("log", file=True, stdOut=True)
        output = trap_stdout(
            logger.log, {'message': 'hello world', 'newline': False})
        self.assertTrue("hello world" in output)
        with open(logger.fileName, 'r') as f:
            logger_text = f.read()
        self.assertTrue("hello world " in logger_text)

    def test_force_to_file(self):
        dogtail.config.config.logDebugToFile = False
        logger = dogtail.logging.Logger("log", file=True, stdOut=False)
        logger.log("hello world", force=True)
        with open(logger.fileName, 'r') as f:
            logger_text = f.read()
        self.assertTrue("hello world" in logger_text)

    def test_results_logger_correct_dict(self):
        logger = dogtail.logging.ResultsLogger("log")
        output = trap_stdout(logger.log, {'entry': {'a': '1'}})
        self.assertEqual('a:      1' in output, True)

    def test_results_logger_incorrect_dict(self):
        logger = dogtail.logging.ResultsLogger("log")
        self.assertRaises(ValueError, logger.log, "not a dict")



#!/usr/bin/env python
# Dogtail demo script
__author__ = 'David Malcolm <dmalcolm@redhat.com>'

# i18n tests

from dogtail.tree import root

import dogtail.i18n
import dogtail.distro

def translateAllStrings(appName):
    """
    Test of the translation functions.

    Take all user-visible strings in an app that's running in the default locale
    and try translating them all into the locale that this script is running in.
    """
    for string in root.application(appName).getUserVisibleStrings():
        print(("User-visible string: %s" % string))
        print(("Translation is:%s"%dogtail.i18n.translate(string)))

print(("Package dependencies: %s"%dogtail.distro.packageDb.getDependencies('evolution')))
#print dogtail.i18n.getMoFilesForPackage('evolution', True)
print(("Translation domains: %s"%dogtail.i18n.getTranslationDomainsForPackage('evolution', True)))

#dogtail.i18n.loadTranslationsFromPackageMoFiles('evolution')
#translateAllStrings('evolution')



#!/usr/bin/env python
__author__ = "Zack Cerza <zcerza@redhat.com>"

from setuptools import setup

def examples():
    import os
    ex_list = os.listdir(os.curdir + '/examples/')
    result = []
    for ex in ex_list:
        if ex.split('.')[-1] == 'py':
            result = result + ['examples/' + ex]
    return result

def examples_data():
    import os
    data_list = os.listdir(os.curdir + '/examples/data/')
    result = []
    for data in data_list:
        result = result + ['examples/data/' + data]
    return result

def tests():
    import os
    ex_list = os.listdir(os.curdir + '/tests/')
    result = []
    for ex in ex_list:
        if ex.split('.')[-1] == 'py':
            result = result + ['tests/' + ex]
    return result

def sniff_icons():
    import os
    list = os.listdir(os.curdir + '/sniff/icons/')
    result = []
    for file in list:
        if file.split('.')[-1] in ('xpm'):
            result = result + ['sniff/icons/' + file]
    return result

def icons(ext_tuple):
    import os
    list = os.listdir(os.curdir + '/icons/')
    result = []
    for file in list:
        if file.split('.')[-1] in ext_tuple:
            result = result + ['icons/' + file]
    return result

def scripts():
    import os
    list = os.listdir(os.curdir + '/scripts/')
    result = ['sniff/sniff']
    for file in list:
        result = result + ['scripts/' + file]
    return result

def session_file():
    result = ['scripts/gnome-dogtail-headless.session']
    return result

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='dogtail',
    version='0.9.11',
    description="GUI test tool and automation framework " +
    "that uses Accessibility (a11y) technologies to " +
    "communicate with desktop applications.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="""Zack Cerza <zcerza@redhat.com>,
Ed Rousseau <rousseau@redhat.com>,
David Malcolm <dmalcolm@redhat.com>,
Vitezslav Humpa <vhumpa@redhat.com>""",
    author_email='dogtail-list@gnome.org',
    url='https://gitlab.com/dogtail/dogtail',
    packages=['dogtail'],
    scripts=scripts(),
    data_files=[
        ('share/doc/dogtail/examples',
        examples()),
        ('share/doc/dogtail/examples/data',
        examples_data()),
        ('share/doc/dogtail/tests',
        tests()),
        ('share/dogtail/glade', ['sniff/sniff.ui']),
        ('share/dogtail/icons', sniff_icons()),
        ('share/applications', ['sniff/sniff.desktop']),
        ('share/icons/hicolor/scalable/apps', icons('svg'))
    ],
    options = {
    'build_scripts': {
        'executable': '/usr/bin/python3',
    }
}
)

# vim: sw=4 ts=4 sts=4 noet ai



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import unittest
import dogtail.version

"""
Unit tests for the dogtail.version module
"""


class TestVersion(unittest.TestCase):
    def test_version_from_string_list(self):
        version = dogtail.version.Version([1, 2, 3])
        self.assertEqual(str(version), "1.2.3")

    def test_version_from_string(self):
        version = dogtail.version.Version.fromString("1.2.3")
        self.assertEqual(str(version), "1.2.3")

    def test_version_less_than(self):
        version = dogtail.version.Version.fromString("1.2.3")
        version_less1 = dogtail.version.Version.fromString("1.2.2")
        version_less2 = dogtail.version.Version.fromString("1.1.3")
        version_less3 = dogtail.version.Version.fromString("0.8.3")
        self.assertTrue(version_less1 < version)
        self.assertTrue(version_less2 < version)
        self.assertTrue(version_less3 < version)
        self.assertTrue(version_less1 <= version)
        self.assertTrue(version_less2 <= version)
        self.assertTrue(version_less3 <= version)

    def test_version_more_than(self):
        version = dogtail.version.Version.fromString("1.2.3")
        version_less1 = dogtail.version.Version.fromString("1.2.2")
        version_less2 = dogtail.version.Version.fromString("1.1.3")
        version_less3 = dogtail.version.Version.fromString("0.8.3")
        self.assertTrue(version > version_less1)
        self.assertTrue(version > version_less2)
        self.assertTrue(version > version_less3)
        self.assertTrue(version >= version_less1)
        self.assertTrue(version >= version_less2)
        self.assertTrue(version >= version_less3)

    def test_version_equals(self):
        version0 = dogtail.version.Version([1, 2, 3])
        version1 = dogtail.version.Version.fromString("1.2.3")
        version2 = dogtail.version.Version.fromString("1.2.2")
        self.assertTrue(version0 == version1)
        self.assertFalse(version0 == version2)
        self.assertFalse(version1 == version2)
        self.assertTrue(version0 >= version1)
        self.assertTrue(version0 <= version1)
        self.assertTrue(version0 >= version2)
        self.assertFalse(version0 <= version2)
        self.assertTrue(version1 >= version2)
        self.assertFalse(version1 <= version2)
        self.assertFalse(version0 != version1)
        self.assertFalse(version1 != version0)
        self.assertTrue(version2 != version0)
        self.assertTrue(version2 != version1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import dogtail.config
import pyatspi
from dogtail.rawinput import absoluteMotion, relativeMotion, doubleClick, press, drag, dragWithTrajectory, \
    pressKey, absoluteMotionWithTrajectory, release, checkCoordinates, click, keyCombo, typeText
from dogtail.tree import SearchError
from gtkdemotest import GtkDemoTest

dogtail.config.config.logDebugToFile = False
dogtail.config.config.logDebugToStdOut = True

"""
Unit tests for the dogtail.rawinput package
"""


class TestRawinput(GtkDemoTest):

    def test_motion(self):
        absoluteMotion(100, 100)
        absoluteMotion(100, 100, mouseDelay=1)
        absoluteMotion(-100, -100, check=False)
        absoluteMotion(100, 100, mouseDelay=1, check=False)
        relativeMotion(-10, -10)
        absoluteMotion(100, 100)
        relativeMotion(-10, -10, mouseDelay=1)

    def test_motion_with_trajectory(self):
        absoluteMotionWithTrajectory(100, 100, 120, 120)
        absoluteMotionWithTrajectory(120, 120, 120, 120)
        absoluteMotionWithTrajectory(120, 120, 130, 120)
        absoluteMotionWithTrajectory(130, 120, 130, 130)
        absoluteMotionWithTrajectory(130, 130, 150, 100, mouseDelay=0.1)

    def test_check_coordinates_direct(self):
        checkCoordinates(0, 0)
        checkCoordinates(-0, -0)
        checkCoordinates(0, 10)
        checkCoordinates(10, 0)
        with self.assertRaises(ValueError):
            checkCoordinates(-1, 100)
        with self.assertRaises(ValueError):
            checkCoordinates(100, -1)
        with self.assertRaises(ValueError):
            checkCoordinates(-1, -1)

    def test_check_coordinates_builtin(self):
        with self.assertRaises(ValueError):
            absoluteMotion(-5, 5)
        with self.assertRaises(ValueError):
            absoluteMotion(5, -5)
        with self.assertRaises(ValueError):
            absoluteMotionWithTrajectory(10, 10, -5, 5)
        with self.assertRaises(ValueError):
            absoluteMotionWithTrajectory(10, 10, 5, -5)
        with self.assertRaises(ValueError):
            absoluteMotionWithTrajectory(-5, 5, 10, 10)
        with self.assertRaises(ValueError):
            absoluteMotionWithTrajectory(5, -5, 10, 10)
        with self.assertRaises(ValueError):
            click(-5, 5)
        with self.assertRaises(ValueError):
            click(5, -5)
        with self.assertRaises(ValueError):
            doubleClick(-5, 5)
        with self.assertRaises(ValueError):
            doubleClick(5, -5)
        with self.assertRaises(ValueError):
            press(-5, 5)
        with self.assertRaises(ValueError):
            press(5, -5)
        with self.assertRaises(ValueError):
            release(-5, 5)
        with self.assertRaises(ValueError):
            release(5, -5)

    def test_doubleClick(self):
        btn = self.app.child('Builder')
        doubleClick(btn.position[0], btn.position[1])
        self.assertEqual(len([x for x in self.app.children if x.name in ['GtkBuilder demo', 'Builder']]), 2)

    def test_click(self):
        btn = self.app.child('Builder')
        self.assertFalse(btn.focused)
        click(btn.position[0], btn.position[1])
        self.assertTrue(btn.focused)

    def test_press_release(self):
        self.runDemo('Builder')
        try:
            wnd = self.app.child('GtkBuilder demo', roleName='frame', recursive=False, retry=False)
        except dogtail.tree.SearchError:
            wnd = self.app.child('Builder', roleName='frame', recursive=False, retry=False)
        btn = wnd.button('New')
        press(btn.position[0] + btn.size[0] / 2, btn.position[1] + btn.size[1] / 2, check=False)
        self.assertIn(pyatspi.STATE_ARMED, btn.getState().getStates())
        release(btn.position[0] + btn.size[0] / 2, btn.position[1] + btn.size[1] / 2, check=False)
        self.assertNotIn(pyatspi.STATE_ARMED, btn.getState().getStates())

    def test_drag(self):
        self.runDemo('Tool Palette')
        win = self.app.window('Tool Palette')
        src = win.findChildren(lambda x: x.roleName == 'push button' and x.showing)[0]
        dst = win.child(roleName='viewport')
        drag((src.position[0] + src.size[0] / 2, src.position[1] + src.size[1] / 2),
             (dst.position[0] + dst.size[0] / 2, dst.position[1] + dst.size[1] / 2))

    def test_drag_with_trajectory(self):
        self.runDemo('Tool Palette')
        win = self.app.window('Tool Palette')
        src = win.findChildren(lambda x: x.roleName == 'push button' and x.showing)[0]
        dst = win.child(roleName='viewport')
        dragWithTrajectory((src.position[0] + src.size[0] / 2, src.position[1] + src.size[1] / 2),
                           (dst.position[0] + dst.size[0] / 2, dst.position[1] + dst.size[1] / 2))

    def test_pressKey_no_such_key(self):
        with self.assertRaises(KeyError):
            pressKey("no such key")

    def test_keyCombo_simple(self):
        keyCombo('<End>')
        self.assertTrue(self.app.child('Tree View').showing)

    def test_keyCombo_multi(self):
        self.runDemo('Clipboard')
        try:
            wnd = self.app.child('Clipboard demo', roleName='frame', retry=False, recursive=False)
        except SearchError:
            wnd = self.app.child('Clipboard', roleName='frame', retry=False, recursive=False)
        textfield = wnd.child(roleName='text')
        textfield.text = 'something'
        keyCombo('<Control>a')
        keyCombo('<Control>c')
        keyCombo('<Control>v')
        keyCombo('<Control>v')
        self.assertEqual(textfield.text, 'somethingsomething')

    def test_keyCombo_wrong_key(self):
        with self.assertRaises(ValueError):
            keyCombo('<WORK_FASTER_THAN_LIGHTSPEED>')

    def test_typeText(self):
        self.runDemo('Clipboard')
        try:
            wnd = self.app.child('Clipboard demo', roleName='frame', retry=False, recursive=False)
        except SearchError:
            wnd = self.app.child('Clipboard', roleName='frame', retry=False, recursive=False)
        textfield = wnd.child(roleName='text')
        typeText('something')
        self.assertEqual(textfield.text, 'something')



#!/usr/bin/env python
# Dogtail demo script using tree.py
# FIXME: Use TC.
__author__ = 'Zack Cerza <zcerza@redhat.com'

from dogtail import tree
from dogtail.utils import run
from dogtail.predicate import GenericPredicate
from time import sleep
from os import environ, path, remove
environ['LANG']='en_US.UTF-8'

# Remove the output file, if it's still there from a previous run
if path.isfile(path.join(path.expandvars("$HOME"), "Desktop", "UTF8demo.txt")):
    remove(path.join(path.expandvars("$HOME"), "Desktop", "UTF8demo.txt"))

# Start gedit.
run("gedit")

# Get a handle to gedit's application object.
gedit = tree.root.application('gedit')

# Get a handle to gedit's text object.
# Last text object is the gedit's text field
textbuffer = gedit.findChildren(GenericPredicate(roleName='text'))[-1]

# This will work only if 'File Browser panel' plugin is disabled
#textbuffer = gedit.child(roleName = 'text')

# Load the UTF-8 demo file.
from sys import path
utfdemo = open(path[0] + '/data/UTF-8-demo.txt', 'r')

# Load the UTF-8 demo file into gedit's text buffer.
textbuffer.text = utfdemo.read()

# Get a handle to gedit's File menu.
filemenu = gedit.menu('File')

# Get a handle to gedit's Save button.
savebutton = gedit.button('Save')

# Click the button
savebutton.click()

# Get a handle to gedit's Save As... dialog.
try:
    saveas = gedit.child(roleName='file chooser')
except tree.SearchError:
    try:
        saveas = gedit.dialog('Save As\\u2026')
    except tree.SearchError:
        saveas = gedit.dialog('Save as...')

# We want to save to the file name 'UTF8demo.txt'.
saveas.child(roleName = 'text').text = 'UTF8demo.txt'

# Save the file on the Desktop

# Don't make the mistake of only searching by name, there are multiple
# "Desktop" entires in the Save As dialog - you have to query for the
# roleName too - see the on-line help for the Dogtail "tree" class for
# details
saveas.child('Desktop', roleName='table cell').click()

#  Click the Save button.
saveas.button('Save').click()

# Let's quit now.
filemenu.click()
filemenu.menuItem('Quit').click()
